;**************************************************************************
;* FileName	: RS485_HW.INC
;* Date 	: 17.02.2004
;* File version : v1.02
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: functions for for rs485 communication
;*		  =====================================
;*		  RS485_SendByte
;*		  RS485_ReceiveByte
;*		  RS485_SendCS
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define	RS485_DATADIR	PORTB,0		; out	(0=Rx, 1=Tx)
;* #define	RS485_TxD	PORTB,1		; out
;* #define	RS485_RxD	PORTB,2		; in
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define RS485_OWN_DATADIR		; If it is defined this module do not control the datadir line.
;*
;* #define DEBUG			; for some debug message
;* #define NO_RS485			; for fast simulation
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  30/12/2003 	Initial Release
;* 1.01   05/01/2004	- correct DATADIR handling
;* 1.02   17/02/2004	- RS485_ReceiveByte dont wait to receive.
;*
;**************************************************************************

;***** VARIABLES **********************************************************

	ifdef INC_RS485_SCS
		cblock
		  RS485_temp
		endc
	endif


;***** SEND SERIAL DATA ***************************************************
;* Send byte
;* Input : W				; byte
;* Output: -
;* Clock : ?
;**************************************************************************
	ifdef INC_RS485
RS485_SendByte	
		ifndef	RS485_OWN_DATADIR
		  bsf	RS485_DATADIR
		endif
		
RS485_waittx	btfss	PIR1, TXIF		; wait for empty buffer
		goto	RS485_waittx

		movwf	TXREG

		ifndef	RS485_OWN_DATADIR
		  bank1
RS485_sb1	  btfss	LL TXSTA, TRMT
		  goto	RS485_sb1
		  bank0
		  bcf	RS485_DATADIR
		endif

		return
	endif



;***** RECEIVE SERIAL DATA ************************************************
;* Receive byte
;* Input : -				; 
;* Output: received byte -> W		;
;*	   staus -> Z			; 1 = ok (jz), 0 = error or dont received (jnz)
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS485
RS485_ReceiveByte

		btfss	PIR1, RCIF
		goto	RS485_norec

		movlw	((1 << OERR) | (1 << FERR))	; serial receive error
		andwf	RCSTA,w
		jz	RS485_rb_no_error

		movf	RCREG,w				; read received byte twice for deletion
		movf	RCREG,w
		
		btfsc	RCSTA, OERR			; if OERR
		bcf	RCSTA, CREN			;    - clear CREN and 
		bsf	RCSTA, CREN			;    - and set again.

RS485_norec	clrw
		clrz					; Z=0 -> error or dont received
		return

RS485_rb_no_error
		movf	RCREG,w
		setz					; Z=1 -> ok
		return
	endif



;***** RS485_SendCS *******************************************************
;* Send constant string
;* Input : -				; 
;* Output: received byte -> W		;
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS485_SCS
RS485_SendCS	movwf	PCLATH

RS485_sendcs1	call	RS485_sendcs_getc
		testw
		skipnz
		return

		call	RS485_SendByte

		incf	RS485_temp,f
		skipnz
		incf	PCLATH,f
		goto	RS485_sendcs1

RS485_sendcs_getc
		movf	RS485_temp,w
		movwf	PCL		; computed goto!

	endif


;**************************************************************************



